﻿using System.Collections.Generic;
using System.Xml;
using System.Xml.Serialization;
using T4UtilityBelt.DataDesigns;
using T4UtilityBelt.Serializers;
using T4UtilityBelt.UxDesigns.Bindings;
using T4UtilityBelt.UxDesigns.Controls;
using T4UtilityBelt.UxDesigns.Specialized;

namespace T4UtilityBelt.UxDesigns.Screens2
{
    public class EntityScreen<TView, TRenderer> : Screen<TView, TRenderer>
        where TView : View<TView, TRenderer>
        where TRenderer : class, T4UtilityBelt.UxDesigns.Screens.Renders.IViewRenderer<TView, TRenderer>
    {
        private Header _header;
        private ActionBar _actionBar;

        public EntityScreen()
        {
            StandardConstructor();
        }

        public EntityScreen(string entityName) 
            
        {
            EntityName = entityName;
            StandardConstructor();
        }

        public EntityScreen(string entityName, string screenName)
        {
            EntityName = entityName;
            Name = screenName;
            StandardConstructor();
        }

        //public override string Name
        //{
        //    get
        //    {
        //        if (string.IsNullOrEmpty(base.Name))
        //        {
        //            return EntityName;
        //        }
        //        return base.Name;
        //    }
        //    set
        //    {
        //        base.Name = value;
        //    }
        //}

        public string EntityName { get; set; }

        [XIgnore]
        public Entity Entity { get; set; }

        [XIgnore]
        public override ControlCollection Controls
        {
            get
            {
                return base.Controls;
            }
        }

        public override IEnumerable<Control> AllControls()
        {
            if (Controls.Count > 0)
            {
                foreach (Control c in Controls)
                {
                    yield return c;
                    foreach (Control cc in c.AllControls())
                    {
                        yield return cc;
                    }
                }
            }
            else
            {
                foreach (var p in Entity.Properties)
                {
                    yield return Control.Create(p);
                }
            }
        }

        private void StandardConstructor()
        {
            Header = new Header() { Text = Name };

            ActionBar = new ActionBar();
        }

        internal override void FixModel(T4UtilityBelt.DataDesigns.EntityModel model)
        {
            base.FixModel(model);

            Entity = model.GetEntity(EntityName);
            if (Entity != null)
            {
                Header.FixModel(Entity);

                ActionBar.FixModel(Entity);
            }
        }


        public Header Header
        {
            get { return _header; }
            set
            {
                Controls.Switch(_header, value);
                _header = value;
            }
        }

        public ActionBar ActionBar
        {
            get { return _actionBar; }
            set
            {
                Controls.Switch(_actionBar, value);
                _actionBar = value;
            }
        }
    }
}